@@ -2,9 +2,11 @@
 namespace PhpAmqpLib\Channel;
 
 use PhpAmqpLib\Connection\AbstractConnection;
+use PhpAmqpLib\Exception\AMQPChannelClosedException;
+use PhpAmqpLib\Exception\AMQPInvalidFrameException;
+use PhpAmqpLib\Exception\AMQPNotImplementedException;
 use PhpAmqpLib\Exception\AMQPOutOfBoundsException;
 use PhpAmqpLib\Exception\AMQPOutOfRangeException;
-use PhpAmqpLib\Exception\AMQPRuntimeException;
 use PhpAmqpLib\Helper\DebugHelper;
 use PhpAmqpLib\Helper\Protocol\MethodMap080;
 use PhpAmqpLib\Helper\Protocol\MethodMap091;
@@ -106,7 +108,7 @@ abstract class AbstractChannel
                 $this->methodMap = new MethodMap080();
                 break;
             default:
-                throw new AMQPRuntimeException(sprintf(
+                throw new AMQPNotImplementedException(sprintf(
                     'Protocol: %s not implemented.',
                     $this->protocolVersion
                 ));
@@ -185,7 +187,7 @@ abstract class AbstractChannel
     public function dispatch($method_sig, $args, $amqpMessage)
     {
         if (!$this->methodMap->valid_method($method_sig)) {
-            throw new AMQPRuntimeException(sprintf(
+            throw new AMQPNotImplementedException(sprintf(
                 'Unknown AMQP method "%s"',
                 $method_sig
             ));
@@ -194,7 +196,7 @@ abstract class AbstractChannel
         $amqp_method = $this->methodMap->get_method($method_sig);
 
         if (!method_exists($this, $amqp_method)) {
-            throw new AMQPRuntimeException(sprintf(
+            throw new AMQPNotImplementedException(sprintf(
                 'Method: "%s" not implemented by class: %s',
                 $amqp_method,
                 get_class($this)
@@ -232,7 +234,7 @@ abstract class AbstractChannel
     protected function send_method_frame($method_sig, $args = '')
     {
         if ($this->connection === null) {
-            throw new AMQPRuntimeException('Channel connection is closed.');
+            throw new AMQPChannelClosedException('Channel connection is closed.');
         }
 
         $this->connection->send_channel_method_frame($this->channel_id, $method_sig, $args);
@@ -319,6 +321,7 @@ abstract class AbstractChannel
      * @param int $timeout
      * @throws \PhpAmqpLib\Exception\AMQPOutOfBoundsException
      * @throws \PhpAmqpLib\Exception\AMQPRuntimeException
+     * @throws \ErrorException
      * @return mixed
      */
     public function wait($allowed_methods = null, $non_blocking = false, $timeout = 0)
@@ -396,7 +399,7 @@ abstract class AbstractChannel
 
     /**
      * @param int $frame_type
-     * @throws \PhpAmqpLib\Exception\AMQPRuntimeException
+     * @throws \PhpAmqpLib\Exception\AMQPInvalidFrameException
      */
     protected function validate_method_frame($frame_type)
     {
@@ -405,7 +408,7 @@ abstract class AbstractChannel
 
     /**
      * @param int $frame_type
-     * @throws \PhpAmqpLib\Exception\AMQPRuntimeException
+     * @throws \PhpAmqpLib\Exception\AMQPInvalidFrameException
      */
     protected function validate_header_frame($frame_type)
     {
@@ -414,7 +417,7 @@ abstract class AbstractChannel
 
     /**
      * @param int $frame_type
-     * @throws \PhpAmqpLib\Exception\AMQPRuntimeException
+     * @throws \PhpAmqpLib\Exception\AMQPInvalidFrameException
      */
     protected function validate_body_frame($frame_type)
     {
@@ -430,7 +433,7 @@ abstract class AbstractChannel
     {
         if ($frameType != $expectedType) {
             $protocolClass = self::$PROTOCOL_CONSTANTS_CLASS;
-            throw new AMQPRuntimeException(sprintf(
+            throw new AMQPInvalidFrameException(sprintf(
                     'Expecting %s, received frame type %s (%s)',
                     $expectedMessage,
                     $frameType,
